<?php

namespace Params\Params\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Params\Params\Http\Requests\ParamsRequest;
use Params\Params\Interfaces\ParamsRepositoryInterface;
use Params\Params\Models\Params;
use Params\Params\Forms\Params as Form;

/**
 * APIController  class for params.
 */
class ParamsAPIController extends BaseController
{

    /**
     * Initialize params resource controller.
     *
     * @param type ParamsRepositoryInterface $params
     *
     * @return null
     */
    public function __construct(ParamsRepositoryInterface $params)
    {
        parent::__construct();
        $this->repository = $params;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Params\Params\Repositories\Criteria\ParamsResourceCriteria::class);
    }

    /**
     * Display a list of params.
     *
     * @return Response
     */
    public function index(ParamsRequest $request)
    {
        return $this->repository
            ->setPresenter(\Params\Params\Repositories\Presenter\ParamsPresenter::class)
            ->paginate();
    }

    /**
     * Display params.
     *
     * @param Request $request
     * @param Model   $params
     *
     * @return Response
     */
    public function show(ParamsRequest $request, Params $params)
    {
        return $params->setPresenter(\Params\Params\Repositories\Presenter\ParamsListPresenter::class);
        ;
    }

    /**
     * Create new params.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ParamsRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('params::params.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('params/params/' . $params->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('params/params');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the params.
     *
     * @param Request $request
     * @param Model   $params
     *
     * @return Response
     */
    public function update(ParamsRequest $request, Params $params)
    {
        try {
            $data = $request->all();

            $params->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('params::params.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('params/params/' . $params->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('params/params/' . $params->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the params.
     *
     * @param Model   $params
     *
     * @return Response
     */
    public function destroy(ParamsRequest $request, Params $params)
    {
        try {
            $params->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('params::params.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('params/params/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('params/params/' . $params->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
